DECLARE 
	v_command VARCHAR2(4000);
	v_mainTableName VARCHAR2(250);
	v_lczTableName VARCHAR2(250);
	v_shortTableName VARCHAR2(250);
	v_lczBackupTableName VARCHAR2(250);
	v_lczOldBackupTableName VARCHAR2(250);
	v_lczColumnsCount NUMBER(8);
	v_lczColumnName VARCHAR2(250);
	v_lczColumnUId VARCHAR2(38);
	v_lczColumnLength NUMBER(8);
	v_lczTableDefinitionTemplate VARCHAR2(4000);
	v_lczColumnDefinitionTemplate VARCHAR2(4000);
	v_lczColumnDefinition VARCHAR2(4000);
	v_primaryCulture VARCHAR2(38);
	v_lczTableNameSuffix VARCHAR2(50) := '';
	v_AllEmployees VARCHAR2(38) := '{A29A3BA5-4B0D-DE11-9A51-005056C00008}';
	v_count NUMBER(8) := 0;
BEGIN
	IF ("tspkg_Update77To78"."fn_IsRowsExist"('SysLic') = 0) THEN
		RETURN;
	END IF;
	v_lczTableDefinitionTemplate := 'CREATE TABLE "##LczBackupTableName##" ("Id" VARCHAR2(38) NOT NULL, "ModifiedOn" TIMESTAMP(6) NULL, "RecordId" VARCHAR2(38) NULL, "SysCultureId" VARCHAR2(38) NULL';
	v_lczColumnDefinitionTemplate := ', "##LczColumnName##" VARCHAR2(##LczColumnLength##)';
	
	BEGIN
		SELECT "GuidValue" into v_primaryCulture
		FROM "SysSettingsValue" v 
			JOIN "SysSettings" s ON v."SysSettingsId" = s."Id"
		WHERE s."Code" = 'PrimaryCulture' AND v."SysAdminUnitId" = v_AllEmployees AND ROWNUM = 1;
		EXCEPTION WHEN NO_DATA_FOUND THEN v_primaryCulture := '';
	END;
	IF v_primaryCulture IS NULL
	THEN
		RETURN;
	END IF;
	DBMS_OUTPUT.PUT_LINE('v_primaryCulture :=' || v_primaryCulture);
	DECLARE CURSOR LczTablesCursor IS 
		SELECT TABLE_NAME
		FROM USER_TABLES ut
			JOIN "SysSchema" ss ON ut.TABLE_NAME = ss."Name"
		WHERE TABLE_NAME LIKE '%Lcz'
			OR TABLE_NAME LIKE '%LczOld';
		BEGIN
			OPEN LczTablesCursor;
			LOOP
				FETCH LczTablesCursor INTO v_lczTableName;
				EXIT WHEN LczTablesCursor%NOTFOUND;
				
				/* Initialize main table name. */
				IF INSTR(v_lczTableName, 'LczOld') > 0
				THEN
					v_lczTableNameSuffix := 'LczOld';
				ELSE
					v_lczTableNameSuffix := 'Lcz';
				END IF;
				v_mainTableName := REPLACE(v_lczTableName, v_lczTableNameSuffix, '');

				DBMS_OUTPUT.PUT_LINE('v_mainTableName := ' || v_mainTableName);

				/* Check if localization table has new structure. */

				IF "tspkg_Update77To78"."fn_HasColumnUId"(v_lczTableName) = 0
				THEN
					DBMS_OUTPUT.PUT_LINE(v_mainTableName || 'New localization table. Do nothing');
					CONTINUE;
				END IF;

				/* Old localization table */

				BEGIN /* Initialize localization columns properties. */

					/* Select original table localization columns properties */

					EXECUTE IMMEDIATE 'DELETE FROM "TableLczColumns"';
					DBMS_OUTPUT.PUT_LINE('v_lczTableName:= ' || v_lczTableName);
					v_command := 'INSERT INTO "TableLczColumns"(
						SELECT 
						ucc.TABLE_NAME "TableName", 
						ucc.COLUMN_NAME "ColumnName",
						"tspkg_Update77To78"."fn_GetColumnUIdFromComment"(ucc.COMMENTS) "ColumnUId",
						utc.DATA_LENGTH "ColumnLength"
						FROM USER_COL_COMMENTS ucc, USER_TAB_COLUMNS utc
						WHERE ucc.TABLE_NAME = ''' || v_mainTableName || ''' 
							AND ucc.TABLE_NAME = utc.TABLE_NAME
							AND ucc.COLUMN_NAME = utc.COLUMN_NAME
							AND "tspkg_Update77To78"."fn_GetColumnUIdFromComment"(ucc.COMMENTS) IN
								(SELECT lczt."ColumnUId" FROM "' || v_lczTableName ||'" lczt))';
					  DBMS_OUTPUT.PUT_LINE(v_command);
					  EXECUTE IMMEDIATE v_command;	

					BEGIN
						SELECT COUNT(*) INTO v_lczColumnsCount FROM "TableLczColumns";
						EXCEPTION WHEN NO_DATA_FOUND THEN v_lczColumnsCount := 0;
					END;
					IF v_lczColumnsCount = 0
					THEN
						DBMS_OUTPUT.PUT_LINE('No lcz columns found');
						"tspkg_Update77To78"."tsp_DropLczTables"(v_lczTableName);
						CONTINUE;
					END IF;

				END;

				BEGIN /* Create old localization table backup. */

					/* Create LczOld backup */

					v_lczOldBackupTableName := v_mainTableName || 'LczOldBackup';
					v_shortTableName := 'BACKUP_' || SUBSTR(SYS_GUID(), 1, 14);
					
					INSERT INTO "BakTableNames"("SourceName", "BackupName", "ShortName") VALUES(v_mainTableName, v_lczOldBackupTableName, v_shortTableName);
					
					v_command := 'CREATE TABLE "' || v_shortTableName || '" AS SELECT *FROM "' || v_lczTableName || '" WHERE "Value" IS NOT NULL';

					DBMS_OUTPUT.PUT_LINE(v_command);
					EXECUTE IMMEDIATE v_command;
          
				END;

				BEGIN /* Create new localization table backup. */

					/*Create Lcz backup */

					v_lczBackupTableName := v_mainTableName || 'LczBackup';
					v_shortTableName := 'BACKUP_' || SUBSTR(SYS_GUID(), 1, 14);

					v_command := REPLACE(v_lczTableDefinitionTemplate, '##LczBackupTableName##', v_shortTableName);
					INSERT INTO "BakTableNames"("SourceName", "BackupName", "ShortName") VALUES(v_mainTableName, v_lczBackupTableName, v_shortTableName);
					
					DECLARE CURSOR LczColumnsCursor IS 
					SELECT "ColumnName", "ColumnLength"
					FROM "TableLczColumns";
					BEGIN
						OPEN LczColumnsCursor;
						LOOP
							FETCH LczColumnsCursor INTO v_lczColumnName, v_lczColumnLength;
							EXIT WHEN LczColumnsCursor%NOTFOUND;
							v_lczColumnDefinition := '';
							v_lczColumnDefinition := REPLACE(v_lczColumnDefinitionTemplate, '##LczColumnLength##', v_lczColumnLength);
							v_lczColumnDefinition := REPLACE(v_lczColumnDefinition, '##LczColumnName##', v_lczColumnName);
							v_command := v_command || v_lczColumnDefinition;	
						END LOOP;
					END;

					v_command := v_command || ')';
					DBMS_OUTPUT.PUT_LINE(v_command);
					EXECUTE IMMEDIATE v_command;

					DECLARE CURSOR LczColumnsCursor IS 
					SELECT "ColumnName", "ColumnUId"
					FROM "TableLczColumns";
					BEGIN
						OPEN LczColumnsCursor;
						LOOP
							FETCH LczColumnsCursor INTO v_lczColumnName, v_lczColumnUId;
							EXIT WHEN LczColumnsCursor%NOTFOUND;
							v_count := 0;
							v_command := 'SELECT COUNT(*) FROM "' || v_shortTableName || '"';
							DBMS_OUTPUT.PUT_LINE(v_command);
							EXECUTE IMMEDIATE v_command INTO v_count;
					    
							v_command := '';
							IF v_count > 0
							THEN
							v_command := 'UPDATE "' || v_shortTableName || '" lcz
								SET lcz."' || v_lczColumnName || '" = (SELECT CAST(SUBSTR(lczOld."Value", 1, ' || v_lczColumnLength || ') 
								as VARCHAR2(' || v_lczColumnLength || '))
									FROM "' || v_lczTableName || '" lczOld
									WHERE lczOld."RecordId" = lcz."RecordId" 
										AND lczOld."SysCultureId" = lcz."SysCultureId" 
										AND lczOld."ColumnUId" = ''' || v_lczColumnUId || ''' AND ROWNUM = 1)
								WHERE EXISTS (
									SELECT 1
									FROM "' || v_lczTableName || '" lczOld
									WHERE lczOld."RecordId" = lcz."RecordId" 
										AND lczOld."SysCultureId" = lcz."SysCultureId" 
										AND lczOld."ColumnUId" = ''' || v_lczColumnUId || ''')';
								DBMS_OUTPUT.PUT_LINE(v_command);
								EXECUTE IMMEDIATE v_command;
								EXECUTE IMMEDIATE 'UPDATE "' || v_shortTableName || '" SET "' || v_lczColumnName || '" = ''''
								  WHERE "' || v_lczColumnName || '" IS NULL';
							ELSE
								v_command := 'INSERT INTO "' || v_shortTableName || '" ("Id", "ModifiedOn", "RecordId", "SysCultureId", "' || v_lczColumnName || '")
								SELECT "tspkg_Utilities"."fn_CreateGuid"(), SYS_EXTRACT_UTC(SYSTIMESTAMP), "RecordId", "SysCultureId", SUBSTR("Value", 1, ' || v_lczColumnLength || ')
								FROM "' || v_lczTableName || '"
								WHERE "ColumnUId" = ''' || v_lczColumnUId || ''' AND "Value" IS NOT NULL';
								DBMS_OUTPUT.PUT_LINE(v_command);
								EXECUTE IMMEDIATE v_command;
							END IF;
						END LOOP;
					END;
				END;

			DBMS_OUTPUT.PUT_LINE('Table dropped : ' || v_lczTableName);
			"tspkg_Update77To78"."tsp_DropLczTables"(v_lczTableName);
			END LOOP;
		END;
		/* Create empty SysModuleLcz */
		IF "tspkg_Update77To78"."fn_IsObjectExist"('SysModuleLcz') = 0 THEN
			EXECUTE IMMEDIATE 'CREATE TABLE "SysModuleLcz" ("Id" VARCHAR2(38 BYTE) NOT NULL)';
		END IF;

END;
